/**
 * Copyright (C) 2012 Ratoc Systems, Inc.
 *
 * Scan Barcode Application for USB60MI
 *
 * Target Barocde Reader: KEYENCE BL-N60
 * 
 */

package com.ratocsystems.usb60sample.scanbarcode;

import jp.ksksue.driver.serial.FTDriver;
import android.hardware.usb.UsbManager;
import android.os.Bundle;
import android.os.Handler;
import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;
import android.support.v4.app.NavUtils;

/**
 * 
 * This is the main Activity
 *
 */

public class Main extends Activity {
	// for Debug
	private static final String TAG = "USB60 - Scanbarcode";
	private static boolean D = true;

	// REX-USB60MI
	FTDriver mUSB60;
	
	// Baudrate is 9600bps
	private int mBaudrate = FTDriver.BAUD9600;
	
	// Status & Barcode View
	private TextView mTvStatus;

	private String mStatusText;
	private String mBCRtext;
	
	// BCR Header & Terminate Code for KEYENCE BL-N60
	private static char StxCode = 0x02, EscCode = 0x1B, 
			EtxCode = 0x03, EotCode = 0x04,
			LFCode = 0x0a, CRCode = 0x0d;
	
	// Thread flag
	private boolean mStop = false;
	
	Handler mHandler = new Handler();
	
	private static final String ACTION_USB_PERMISSION =
			"sanbarcode.action.USB_PERMISSION";
		
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (D) Log.e(TAG, "+++ ON CREATE +++");
        
        // Set up the window layout
        setContentView(R.layout.main);
        
        mTvStatus = (TextView)findViewById(R.id.tvStatus);
        mTvStatus.setTextSize(40);
        
        // Get Class Driver (FTDriver) for FTDI USB Serial Convertor
        mUSB60 = new FTDriver((UsbManager)getSystemService(Context.USB_SERVICE));
        
        IntentFilter filter = new IntentFilter();
        filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
        filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
        registerReceiver(mUsbReceiver, filter);
        
        PendingIntent permissionIntent = PendingIntent.getBroadcast(this,  0,
        		new Intent(ACTION_USB_PERMISSION), 0);
        mUSB60.setPermissionIntent(permissionIntent);
        
        if (mUSB60.begin(mBaudrate)) {
        	mainloop();
        } else {
            if (D) Log.e(TAG, "+++ Not Connected +++");
        	mTvStatus.setText(R.string.Notconnected);
        }
        
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }
    
    @Override
	protected void onDestroy() {
    	mUSB60.end();
    	mStop = true;
    	unregisterReceiver(mUsbReceiver);
    	
		super.onDestroy();
	}

    private void mainloop() {
    	mStop = false;
    	mTvStatus.setText(R.string.Connected);
    	// Start thread for scannig Barcode
    	new Thread(mScanBcrLoop).start();
    }
    
    //
    // Scan Barcode loop
    //
    
    private Runnable mScanBcrLoop = new Runnable() {
    	@Override
    	public void run() {
    		
    		int len;
    		byte rChar;
    		// Receive buffer
    		byte[] rbuf = new byte[4096];
    		int rbufp;
    		
    		mStatusText = "";
			mBCRtext = "";
    		
    		while (true) {
    			// get receive data
    			len = mUSB60.read(rbuf);
    			
    			if (len > 0) {
    				
    				for (rbufp=0; rbufp < len; rbufp++) {
    					
    					rChar = (byte)(rbuf[rbufp] & 0x007F);

    					if (rChar == StxCode || rChar == EscCode) {
    	    				mBCRtext = "";
    	    				continue; // Skip header code
    					}
    					
    					// Detect terminate code
    					if (rChar == EtxCode
    							|| rChar == EotCode
    							|| rChar == LFCode
    							|| rChar == CRCode) {
//    	            		if (D) Log.e(TAG, mBCRtext);

    	    				mStatusText = String.format("%3d Digits [", mBCRtext.length()) + mBCRtext + "]";
    	    				mHandler.post(new Runnable() {
    	    					public void run() {
    	    						mTvStatus.setText(mStatusText); // Display Status & Barcode
    	    					}
    	    				});
    	    				
    	    	            if (D) {
    	    	            	if (rbufp < len-1) {
    	    	            		mStatusText = String.format("+++ Remain %03d bytes +++",
    	    	            												(len - rbufp)-1);
    	    	            		Log.e(TAG, mStatusText);
    	    	            	}
    	    	            }
    	    				mBCRtext = "";
    					} else if (rChar == 0x0000) {
        						continue;
    					} else {
 							mBCRtext = mBCRtext + String.format("%C",  rChar);
    					}
    				}
    				
    			}
    			
    			try {
    				Thread.sleep(50);
    			} catch (InterruptedException e) {
    				e.printStackTrace();
    			}
    			
    			if (mStop)
    				return;
    		}
    	}
    };
    
    
	//
    // Broardcast Receiver for USB60 Hotplug insert/remove, detect
    //
    BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {
    	@Override
    	public void onReceive(Context context, Intent intent) {
    		String action = intent.getAction();
    		
    		//
    		// Insert
    		//
    		if (UsbManager.ACTION_USB_DEVICE_ATTACHED.equals(action)) {
    			if (!mUSB60.isConnected()) {
                    if (D) Log.e(TAG, "+++ Insert +++");
    				mUSB60.usbAttached(intent);
    				mUSB60.begin(mBaudrate);
    				mainloop();
    			}
    			
        		//
        		// Remove
        		//
    		} else if (UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action)) {
                if (D) Log.e(TAG, "+++ Removed +++");
    			mTvStatus.setText(R.string.Notconnected);
    			mUSB60.usbDetached(intent);
    			mUSB60.end();
    			mStop = true;
    			
        		//
        		// Detect
        		//
    		} else if (ACTION_USB_PERMISSION.equals(action)) {
    			synchronized(this) {
    				if (!mUSB60.isConnected()) {
    					mUSB60.usbAttached(intent);
    					mUSB60.begin(mBaudrate);
    				}
    			}
    			mainloop();
    		}
    	}
    };

}
